
import javax.swing.JOptionPane;

/**
 * Uma classe para testar interactivamente um videoDisk.
 * 
 * @author Vasco T. Vasconcelos
 * @version 1.0 16/Set/2010 
 */
public class Interact {

    /**
     * O gravador
     */
    private GeneralVideoDisk disk;

    /**
     * As opcoes disponiveis no menu
     */
    private String[] choices = 
    	{ "gravar", "apagar", "durao", "ttulo", "ndice", "sair" };

    /**
     * Construtor
     * 
     * @param length - A durao da cassete
     */
    public Interact(int length) {
        this.disk = new GeneralVideoDisk(length);
    }

    /**
     * Correr o teste
     */
    public void run() {
        int choiceNum = 0;
        while (choiceNum < choices.length - 1) {
            choiceNum = getNextOperation();
            try {
                switch (choiceNum) {
                    case 0:
                        record();
                        break;
                    case 1:
                        erase();
                        break;
                    case 2:
                        duration();
                        break;
                    case 3:
                        title();
                        break;
                    case 4:
                        indexOf();
                        break;
                    case 5:
                        break;
                    default:
                        JOptionPane.showMessageDialog(null, "Opo invlida");
                }
            } catch (Exception e) {
                JOptionPane.showMessageDialog(null, e.getMessage()
                        + "\nTente novamente", "Erro",
                        JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    /**
     * A operao seguinte, lida do teclado
     * 
     * @return int que representa a escolha do utilizador
     */
    private int getNextOperation() {
        return JOptionPane.showOptionDialog(null, "Tempo restante: "
                + disk.remainingTime() + "/" + disk.capacity()
                + "; ltimo ndice: " + disk.lastIndex()
                + (disk.isEmpty() ? " (vazio) " : "") + "\nEscolha uma opo",
                "Vdeo-gravador AED, para mim e pra voc",
                JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE,
                null, choices, choices[0]);
    }

    /**
     * Grava um programa com o nome e duracao lido do teclado
     * 
     * @throws Exception - Quando a duracao introduzida for invalida
     */
    private void record() throws Exception {
        String name = readTitle();
        int len = readLength();
        Program prog = new Program(name, len);
        disk.record(prog);
        JOptionPane.showMessageDialog(null, "Gravao " + name + " completa");
    }

    /**
     * Apaga um programa lido do teclado
     * 
     * @throws Exception - Quando o indice introduzido for invalido
     */
    private void erase() throws Exception {
        int index = readIndex();
        Program prog = disk.recording(index);
        disk.erase(prog);
        JOptionPane.showMessageDialog(null, "Programa " + index + " apagado");
    }

    /**
     * Apresenta a duracao de um programa cujo indice eh lido do teclado
     * 
     * @throws Exception - Quando o indice introduzido for invalido
     */
    private void duration() throws Exception {
        int index = readIndex();
        Program prog = disk.recording(index);
        JOptionPane.showMessageDialog(null, "O programa ndice " + index
                + " tem durao " + disk.duration(prog));
    }

    /**
     * Apresenta o titulo de um programa cujo indice eh lido do teclado
     * 
     * @throws Exception - Quando o indice introduzido for invalido
     */
    private void title() throws Exception {
        int index = readIndex();
        Program prog = disk.recording(index);
        JOptionPane.showMessageDialog(null, "O programa ndice " + index
                + " tem o ttulo " + disk.title(prog));
    }

    /**
     * Apresenta o indice do nome de um programa cujo nome eh lido do teclado
     */
    private void indexOf() {

        String title = JOptionPane.showInputDialog("Nome do programa");
        Program prog = new Program(title, 0);
        JOptionPane.showMessageDialog(null, "O programa " + title
                + " tem o ndice " + disk.indexOf(prog));
    }

    /**
     * O titulo de um programa, lido do teclado
     * 
     * @return Titulo do programa
     */
    private String readTitle() {
        return JOptionPane.showInputDialog("Ttulo do programa");
    }

    /**
     * O indice de um programa, lido do teclado
     * 
     * @return indice do programa
     * @throws Exception - Quando o indice introduzido for invalido
     */
    private int readIndex() throws Exception {
        try {
            int i = Integer.parseInt(JOptionPane
                    .showInputDialog("ndice do programa"));
            if (i < 0 || i > disk.lastIndex())
                throw new Exception();
            return i;
        } catch (Exception e) {
            throw new Exception("Preciso de um ndice entre " + 0 + " e "
                    + disk.lastIndex());
        }
    }

    /**
     * A duracao de um programa, lida do teclado
     * 
     * @return duracao do programa
     * @throws Exception Quando a duraao introduzida for invalida
     */
    private int readLength() throws Exception {
        try {
            int i = Integer.parseInt(JOptionPane
                    .showInputDialog("Durao do programa a gravar"));
            if (i < 0 || i > disk.remainingTime())
                throw new Exception();
            return i;
        } catch (Exception e) {
            throw new Exception("Preciso de uma durao entre " + 0 + " e "
                    + disk.remainingTime());
        }
    }
}
