/**
 * Classe que representa um programa numa cassete video
 * 
 * @author Vasco T. Vasconcelos
 * @version 1.1 19/Jul/2013 as 
 */
public class Program {

	/**
	 * Titulo do programa
	 */
    private String title;

    /**
     * Duracao do programa
     */
    private int length;

    /**
     * Construtor 
     * 
     * @param title - titulo do programa
     * @param length - duracao do programa
     */
    public Program(String title, int length) {
        this.title = title;
        this.length = length;
    }

    /**
     * A duraao do programa
     * 
     * @return a duracao do programa
     */
    public int getLength() {
        return length;
    }

    /**
     * O titulo do programa
     * 
     * @return o titulo do programa
     */
    public String getTitle() {
        return title;
    }

    /**
     * O programa eh vazio, i.e., tem duracao 0?
     * 
     * @return true se o programa tiver duracao 0, false caso contrario
     */
    public boolean isEmpty() {
        return length == 0;
    }

    /**
     * Devolve o hash code para este Program
     * 
     * @return o hash code para este programa
     */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + length;
		result = prime * result + ((title == null) ? 0 : title.hashCode());
		return result;
	}

    /**
     * Este objeto eh igual ao dado?
     * 
     * @param o - o objeto a comparar
     * @return true se os objetos forem iguais, false caso contrario
     */
    @Override
    public boolean equals(Object o) {
        return this == o || o != null && o instanceof Program
                && ((Program) o).title.equals(title)
                && ((Program) o).length == length;
    }
    
    /**
     * A representacao textual de Program
     * 
     * @return String que representa este Program
     */
    @Override
    public String toString() {
    	return title + ": " + length;
    }
}
