import java.util.Random;

/**
 * Uma classe para testar automaticamente um VideoDisk.
 * 
 * @author Ana Santos
 * @version 1.0 19/Jul/2013 
 */
public class RandomVideoDiskTest {

	/**
	 * O gravador
	 */
	private GeneralVideoDisk disk;

	/**
	 * Numero de operacoes disponiveis
	 */
	private final static int NUM_OPS = 7;

	/**
	 * Objeto random
	 */
	private Random rand;

	/**
	 * Possiveis titulos para programas
	 */
	private String[] nomes = {"Prog 1", "Prog 2", "Prog 3", "Prog 4", "Prog 5", 
			"Prog 6", "Prog 7", "Prog 8", "Prog 9", "Prog 10"};

	/**
	 * Numero de operacoes a testar
	 */
	private int howMany;

	/**
	 * Construtor
	 * 
	 * @param length - A duracao do VideoDisk
	 * @param howMany - numero de operacoes a efetuar no teste
	 */
	public RandomVideoDiskTest(int length, int howMany) {
		this.disk = new GeneralVideoDisk(length);
		rand = new Random();
		this.howMany = howMany;
	}
	/**
	 * Correr o teste
	 */
	public void run() {
		
		for (int i = 0; i < howMany; i++) {
			
			switch (rand.nextInt(NUM_OPS)) {
			case 0:
				record();
				break;
			case 1:
				erase();
				break;
			case 2:
				duration();
				break;
			case 3:
				title();
				break;
			case 4:
				indexOf();
				break;
			case 5:
				System.out.println("Capacidade deste VideoDisk: " + disk.capacity() + 
						". Tempo restante: " + disk.remainingTime());
				break;
			case 6:
				System.out.println("Este VideoDisk" + (disk.isEmpty()? " " : " nao ")
						+ "esta vazio.");
				break;
			}
		}
	}

	/**
	 * Grava um programa 
	 */
	private void record() {

		String name = getRandomString();
		int len = getRandomLength();
		Program prog = new Program(name, len);
		try {
			disk.record(prog);
			System.out.println("Gravado programa " + name + " com duracao " + len);
		} catch (Exception e) {
			System.out.println ("Duracao " + len + "invalida. VideoDisk so tem " +
					disk.remainingTime() + " tempo restante");
		}
	}

	/**
	 * Apaga um programa
	 */
	private void erase() {
		int index = getRandomIndex();
		try {
			Program prog = disk.recording(index);
			disk.erase(prog);   
			System.out.println("Apagado programa no index " + index + ": " + prog.getTitle());
		} catch (Exception e) {
			System.out.println ("Index " + index + " invalido. " +
					"VideoDisk " + (disk.isEmpty()? "esta vazio" : "so tem " +
					(disk.lastIndex() + 1) + " programas"));
		}
	}
	
	/**
	 * Apresenta a duracao de um programa
	 */
	private void duration() {
		int index = getRandomIndex();
		try {
			Program prog = disk.recording(index);
			System.out.println("O programa ndice " + index + 
					(prog.isEmpty()? " esta vazio " :  " tem durao " + disk.duration(prog)));

		} catch (Exception e) {
			System.out.println ("Index " + index + " invalido. " +
					"VideoDisk " + (disk.isEmpty()? "esta vazio" : "so tem " +
					(disk.lastIndex() + 1) + " programas"));
		}
	}
	
	/**
	 * Apresenta o titulo de um programa
	 */
	private void title() {
		int index = getRandomIndex();
		try {
			Program prog = disk.recording(index);
			System.out.println("O programa com indice " + index
					+ " tem o titulo: " + disk.title(prog));
		} catch (Exception e) {
			System.out.println ("Index " + index + " invalido. " +
					"VideoDisk " + (disk.isEmpty()? "esta vazio" : "so tem " +
					(disk.lastIndex() + 1) + " programas"));
		}
	}

	/**
	 * Apresenta o indice do nome de um programa
	 */
	private void indexOf() {
		String title = getRandomString();
		Program prog = new Program(title, 0);
		System.out.println("O programa " + title
				+ " tem o ndice " + disk.indexOf(prog));
	}

	/**
	 * Devolve um dos nomes possiveis
	 * 
	 * @return um nome aleatorio dentro dos possiveis
	 */
	private String getRandomString() {
		return nomes[rand.nextInt(nomes.length)];
	}

	/**
	 * Devolve um indice. Pode ser invalido
	 * 
	 * @return um indice
	 */
	private int getRandomIndex() {
		return rand.nextInt(disk.lastIndex()+2);
	}

	/**
	 * Devolve uma duracao.
	 * 
	 * @return uma duracao
	 */
	private int getRandomLength() {
		return rand.nextInt(disk.remainingTime());
	}
}
